@@ -1,25 +1,25 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2013 Joe Wreschnig, Michael Urman, Iñigo Serna,
+# Copyright 2004-2018 Joe Wreschnig, Michael Urman, Iñigo Serna,
 #                     Christoph Reiter, Steven Robertson, Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
-
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, GLib
 
 from quodlibet import config
 from quodlibet import qltk
-
-from quodlibet.browsers._base import Browser
-from quodlibet.query import Query
+from quodlibet import _
+from quodlibet.browsers import Browser
 from quodlibet.qltk.ccb import ConfigCheckMenuItem
 from quodlibet.qltk.completion import LibraryTagCompletion
 from quodlibet.qltk.menubutton import MenuButton
-from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.searchbar import LimitSearchBarBox
+from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.x import Align, SymbolicIconImage
+from quodlibet.qltk import Icons
 
 
 class PreferencesButton(Gtk.HBox):
@@ -35,21 +35,19 @@ class PreferencesButton(Gtk.HBox):
         menu.show_all()
 
         button = MenuButton(
-            SymbolicIconImage("emblem-system", Gtk.IconSize.MENU),
+            SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.MENU),
             arrow=True)
         button.set_menu(menu)
         self.pack_start(button, True, True, 0)
 
 
-class SearchBar(Gtk.VBox, Browser):
-    """Like EmptyBar, but the user can also enter a query manually"""
-
-    __gsignals__ = Browser.__gsignals__
+class SearchBar(Browser):
+    """A browser in which queries are parsed and used to filter results"""
 
     name = _("Search Library")
     accelerated_name = _("_Search Library")
+    keys = ["SearchBar"]
     priority = 1
-    in_menu = True
 
     def pack(self, songpane):
         container = Gtk.VBox(spacing=6)
@@ -64,6 +62,7 @@ class SearchBar(Gtk.VBox, Browser):
     def __init__(self, library):
         super(SearchBar, self).__init__()
         self.set_spacing(6)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
 
         self._query = None
         self._library = library
@@ -101,13 +100,8 @@ class SearchBar(Gtk.VBox, Browser):
         qltk.get_top_parent(widget).songlist.grab_focus()
 
     def _get_songs(self):
-        text = self._get_text()
-        try:
-            self._query = Query(text, star=SongList.star)
-        except Query.error:
-            pass
-        else:
-            return self._query.filter(self._library)
+        self._query = self._sb_box.get_query(SongList.star)
+        return self._query.filter(self._library) if self._query else None
 
     def activate(self):
         songs = self._get_songs()
@@ -119,14 +113,10 @@ class SearchBar(Gtk.VBox, Browser):
         self.activate()
 
     def save(self):
-        config.set("browsers", "query_text", self._get_text())
+        config.settext("browsers", "query_text", self._get_text())
 
     def restore(self):
-        try:
-            text = config.get("browsers", "query_text")
-        except config.Error:
-            return
-
+        text = config.gettext("browsers", "query_text")
         self._set_text(text)
 
     def finalize(self, restore):
@@ -139,6 +129,9 @@ class SearchBar(Gtk.VBox, Browser):
         self._set_text(text)
         self.activate()
 
+    def get_filter_text(self):
+        return self._get_text()
+
     def unfilter(self):
         self.filter_text("")
 
