@@ -1,18 +1,25 @@
+# -*- coding: utf-8 -*-
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
 import re
 
 from gi.repository import Gtk, GObject
 
+from quodlibet import _
 from quodlibet.plugins.editing import RenameFilesPlugin, TagsFromPathPlugin
 from quodlibet.util import connect_obj
+from quodlibet.qltk import Icons
 
 
 class RegExpSub(Gtk.HBox, RenameFilesPlugin, TagsFromPathPlugin):
     PLUGIN_ID = "Regex Substitution"
     PLUGIN_NAME = _("Regex Substitution")
-    PLUGIN_DESC = _("Allow arbitrary regex substitutions (s///) when "
+    PLUGIN_DESC = _("Allows arbitrary regex substitutions (s///) when "
                     "tagging or renaming files.")
-    PLUGIN_ICON = Gtk.STOCK_FIND_AND_REPLACE
-    PLUGIN_VERSION = "1"
+    PLUGIN_ICON = Icons.EDIT_FIND_REPLACE
 
     __gsignals__ = {
         "changed": (GObject.SignalFlags.RUN_LAST, None, ())
@@ -33,8 +40,8 @@ class RegExpSub(Gtk.HBox, RenameFilesPlugin, TagsFromPathPlugin):
         connect_obj(self._to, 'changed', self.emit, 'changed')
 
     def filter(self, orig_or_tag, value):
-        fr = self._from.get_text().decode('utf-8')
-        to = self._to.get_text().decode('utf-8')
+        fr = self._from.get_text()
+        to = self._to.get_text()
         try:
             return re.sub(fr, to, value)
         except:
