@@ -1,51 +1,68 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#           2012-2014 Nick Boultbee
+#           2012-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 # Much of this code is highly optimized, because many of the functions
 # are called in tight loops. Don't change things just to make them
 # more readable, unless they're also faster.
 
 import os
+import re
 import shutil
 import time
+from collections import OrderedDict
+from itertools import zip_longest
 
-from ._image import ImageContainer
+from senf import fsn2uri, fsnative, fsn2text, devnull, bytes2fsn, path2fsn
 
-from quodlibet import const
+from quodlibet import _, print_d
 from quodlibet import util
 from quodlibet import config
-from quodlibet.util.path import mkdir, fsdecode, mtime, expanduser, is_fsnative
-from quodlibet.util.path import normalize_path, fsnative, escape_filename
-from quodlibet.util.string import encode
+from quodlibet.util.path import mkdir, mtime, expanduser, normalize_path, \
+                                ismount, get_home_dir, RootPathFile
+from quodlibet.util.string import encode, decode, isascii
+from quodlibet.util.environment import is_windows
 
-from quodlibet.util.uri import URI
+from quodlibet.util import iso639
 from quodlibet.util import human_sort_key as human, capitalize
 
 from quodlibet.util.tags import TAG_ROLES, TAG_TO_SORT
 
+from ._image import ImageContainer
+from ._misc import AudioFileError, translate_errors
 
-MIGRATE = set(["~#playcount", "~#laststarted", "~#lastplayed", "~#added",
-               "~#skipcount", "~#rating", "~bookmark"])
+
+translate_errors
+
+MIGRATE = {"~#playcount", "~#laststarted", "~#lastplayed", "~#added",
+           "~#skipcount", "~#rating", "~bookmark"}
 """These get migrated if a song gets reloaded"""
 
 PEOPLE = ["artist", "albumartist", "author", "composer", "~performers",
           "originalartist", "lyricist", "arranger", "conductor"]
 """Sources of the ~people tag, most important first"""
 
-INTERN_NUM_DEFAULT = set(
-    ["~#lastplayed", "~#laststarted", "~#playcount",
-     "~#skipcount", "~#length", "~#bitrate", "~#filesize"])
-"""Default to 0"""
+TIME_TAGS = {"~#lastplayed", "~#laststarted", "~#added", "~#mtime"}
+"""Time in seconds since epoch, defaults to 0"""
 
-FILESYSTEM_TAGS = set(["~filename", "~basename", "~dirname"])
+SIZE_TAGS = {"~#filesize"}
+"""Size in bytes, defaults to 0"""
+
+NUMERIC_ZERO_DEFAULT = {"~#skipcount", "~#playcount", "~#length", "~#bitrate"}
+"""Defaults to 0"""
+
+NUMERIC_ZERO_DEFAULT.update(TIME_TAGS)
+NUMERIC_ZERO_DEFAULT.update(SIZE_TAGS)
+
+FILESYSTEM_TAGS = {"~filename", "~basename", "~dirname", "~mountpoint"}
 """Values are bytes in Linux instead of unicode"""
 
-SORT_TO_TAG = dict([(v, k) for (k, v) in TAG_TO_SORT.iteritems()])
+SORT_TO_TAG = dict([(v, k) for (k, v) in TAG_TO_SORT.items()])
 """Reverse map, so sort tags can fall back to the normal ones"""
 
 PEOPLE_SORT = [TAG_TO_SORT.get(k, k) for k in PEOPLE]
@@ -55,32 +72,72 @@ VARIOUS_ARTISTS_VALUES = 'V.A.', 'various artists', 'Various Artists'
 """Values for ~people representing lots of people, most important last"""
 
 
+def decode_value(tag, value):
+    """Returns a unicode representation of the passed value, based on
+    the type and the tag it originated from.
+
+    Not reversible.
+    """
+
+    if tag in FILESYSTEM_TAGS:
+        return fsn2text(value)
+    elif tag[:2] == "~#":
+        if isinstance(value, float):
+            return u"%.2f" % value
+        else:
+            return str(value)
+    return str(value)
+
+
 class AudioFile(dict, ImageContainer):
     """An audio file. It looks like a dict, but implements synthetic
     and tied tags via __call__ rather than __getitem__. This means
     __getitem__, get, and so on can be used for efficiency.
 
     If you need to sort many AudioFiles, you can use their sort_key
-    attribute as a decoration."""
+    attribute as a decoration.
+
+    Keys are either ASCII str or unicode.
+    Values are always unicode except if the tag is part of FILESYSTEM_TAGS,
+    then the value is of the path type (str on UNIX, unicode on Windows)
+
+    Some methods will make sure the returned values are always unicode, see
+    their description.
+    """
 
-    # New tags received from the backend will update the song
     fill_metadata = False
-    # New song duration from the backend will update the song
+    """New tags received from the backend will update the song"""
+
     fill_length = False
-    # Container for multiple songs, while played new songs can start/end
+    """New song duration from the backend will update the song"""
+
     multisong = False
-    # Part of a multisong
+    """Is a container for multiple songs
+     i.e. while played new songs can start / end"""
+
     streamsong = False
-    # Can be added to the queue, playlists
+    """Is part of a multisong"""
+
     can_add = True
-    # Is a real file
+    """Can be added to the queue, playlists"""
+
     is_file = True
+    """Is a real (local) file"""
 
     format = "Unknown Audio File"
+    """The underlying file format"""
+
     mimes = []
+    """MIME types this class can represent"""
+
+    def __init__(self, default=tuple(), **kwargs):
+        for key, value in dict(default).items():
+            self[key] = value
+        for key, value in kwargs.items():
+            self[key] = value
 
     def __song_key(self):
-        return (self("~#disc"), self("~#track"),
+        return (self("~#disc", 1), self("~#track", 1),
             human(self("artistsort")),
             self.get("musicbrainz_artistid", ""),
             human(self.get("title", "")),
@@ -109,9 +166,9 @@ class AudioFile(dict, ImageContainer):
         elif tag == "artistsort":
             return artist_sort
         elif tag in FILESYSTEM_TAGS:
-            return lambda song: fsdecode(song(tag), note=False)
+            return lambda song: fsn2text(song(tag))
         elif tag.startswith("~#") and "~" not in tag[2:]:
-            return lambda song: song(tag)
+            return lambda song: song(tag, 0)
         return lambda song: human(song(tag))
 
     def __getstate__(self):
@@ -123,17 +180,29 @@ class AudioFile(dict, ImageContainer):
         pass
 
     def __setitem__(self, key, value):
+        # validate key
+        if not isinstance(key, str):
+            raise TypeError("key has to be str")
+
+        # validate value
+        if key.startswith("~#"):
+            if not isinstance(value, (int, float)):
+                raise TypeError
+        elif key in FILESYSTEM_TAGS:
+            if not isinstance(value, fsnative):
+                value = path2fsn(value)
+        else:
+            value = str(value)
+
         dict.__setitem__(self, key, value)
-        if not self.__dict__:
-            return
+
         pop = self.__dict__.pop
         pop("album_key", None)
         pop("sort_key", None)
 
     def __delitem__(self, key):
         dict.__delitem__(self, key)
-        if not self.__dict__:
-            return
+
         pop = self.__dict__.pop
         pop("album_key", None)
         pop("sort_key", None)
@@ -146,14 +215,6 @@ class AudioFile(dict, ImageContainer):
     def mountpoint(self):
         return self["~mountpoint"]
 
-    def __cmp__(self, other):
-        if not other:
-            return -1
-        try:
-            return cmp(self.sort_key, other.sort_key)
-        except AttributeError:
-            return -1
-
     def __hash__(self):
         # Dicts aren't hashable by default, so we need a hash
         # function. Previously this used ~filename. That created a
@@ -165,13 +226,21 @@ class AudioFile(dict, ImageContainer):
         # And to preserve Python hash rules, we need a strict __eq__.
         return self is other
 
+    def __lt__(self, other):
+        return self.sort_key < other.sort_key
+
     def __ne__(self, other):
         return self is not other
 
     def reload(self):
-        """Reload an audio file from disk. The caller is responsible for
-        handling any errors."""
+        """Reload an audio file from disk. If reloading fails nothing will
+        change.
 
+        Raises:
+            AudioFileError: if the file fails to load
+        """
+
+        backup = dict(self)
         fn = self["~filename"]
         saved = {}
         for key in self:
@@ -179,14 +248,19 @@ class AudioFile(dict, ImageContainer):
                 saved[key] = self[key]
         self.clear()
         self["~filename"] = fn
-        self.__init__(fn)
-        self.update(saved)
+        try:
+            self.__init__(fn)
+        except AudioFileError:
+            self.update(backup)
+            raise
+        else:
+            self.update(saved)
 
     def realkeys(self):
         """Returns a list of keys that are not internal, i.e. they don't
         have '~' in them."""
 
-        return filter(lambda s: s[:1] != "~", self.keys())
+        return list(filter(lambda s: s[:1] != "~", self.keys()))
 
     def prefixkeys(self, prefix):
         """Returns a list of dict keys that either match prefix or start
@@ -204,28 +278,42 @@ class AudioFile(dict, ImageContainer):
         return "\n".join(self.list_unique(sorted(self.prefixkeys(tag))))
 
     def iterrealitems(self):
-        return ((k, v) for (k, v) in self.iteritems() if k[:1] != "~")
+        return ((k, v) for (k, v) in self.items() if k[:1] != "~")
 
-    def __call__(self, key, default=u"", connector=" - "):
-        """Return a key, synthesizing it if necessary. A default value
-        may be given (like dict.get); the default default is an empty
-        unicode string (even if the tag is numeric).
+    def __call__(self, key, default=u"", connector=" - ", joiner=', '):
+        """Return the value(s) for a key, synthesizing if necessary.
+        Multiple values for a key are delimited by newlines.
 
-        If a tied tag ('a~b') is requested, the 'connector' keyword
+        A default value may be given (like `dict.get`);
+        the default default is an empty unicode string
+        (even if the tag is numeric).
+
+        If a tied tag ('a~b') is requested, the `connector` keyword
         argument may be used to specify what it is tied with.
+        In case the tied tag contains numeric and file path tags, the result
+        will still be a unicode string.
+        The `joiner` keyword specifies how multiple *values* will be joined
+        within that tied tag output, e.g.
+            ~people~title = "Kanye West, Jay Z - New Day"
+
+        For details on tied tags, see the documentation for `util.tagsplit`.
+        """
 
-        For details on tied tags, see the documentation for util.tagsplit."""
         if key[:1] == "~":
             key = key[1:]
             if "~" in key:
-                # FIXME: decode ~filename etc.
-                if not isinstance(default, basestring):
-                    return default
-                return connector.join(
-                    filter(None,
-                    map(lambda x: isinstance(x, basestring) and x or str(x),
-                    map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
-                    map(self.__call__, util.tagsplit("~" + key)))))) or default
+                real_key = "~" + key
+                values = []
+                sub_tags = util.tagsplit(real_key)
+                # If it's genuinely a tied tag (not ~~people etc), we want
+                # to delimit the multi-values separately from the tying
+                j = joiner if len(sub_tags) > 1 else "\n"
+                for t in sub_tags:
+                    vs = [decode_value(real_key, v) for v in (self.list(t))]
+                    v = j.join(vs)
+                    if v:
+                        values.append(v)
+                return connector.join(values) or default
             elif key == "#track":
                 try:
                     return int(self["tracknumber"].split("/")[0])
@@ -287,9 +375,26 @@ class AudioFile(dict, ImageContainer):
                 try:
                     return self["~uri"]
                 except KeyError:
-                    return URI.frompath(self["~filename"])
+                    return fsn2uri(self["~filename"])
             elif key == "format":
-                return self.get("~format", self.format)
+                return self.get("~format", str(self.format))
+            elif key == "codec":
+                codec = self.get("~codec")
+                if codec is None:
+                    return self("~format")
+                return codec
+            elif key == "encoding":
+                parts = filter(None,
+                               [self.get("~encoding"), self.get("encodedby")])
+                encoding = u"\n".join(parts)
+                return encoding or default
+            elif key == "language":
+                codes = self.list("language")
+                if not codes:
+                    return default
+                return u"\n".join(iso639.translate(c) or c for c in codes)
+            elif key == "bitrate":
+                return util.format_bitrate(self("~#bitrate"))
             elif key == "#date":
                 date = self.get("date")
                 if date is None:
@@ -320,11 +425,21 @@ class AudioFile(dict, ImageContainer):
                 except (ValueError, IndexError, TypeError, KeyError):
                     return default
             elif key == "lyrics":
+                # First, try the embedded lyrics.
                 try:
-                    fileobj = file(self.lyric_filename, "rU")
+                    return self[key]
+                except KeyError:
+                    pass
+
+                # If there are no embedded lyrics, try to read them from
+                # the external file.
+                fn = self.lyric_filename
+                try:
+                    fileobj = open(fn, "rb")
                 except EnvironmentError:
                     return default
                 else:
+                    print_d("Reading lyrics from %s" % fn)
                     return fileobj.read().decode("utf-8", "replace")
             elif key == "filesize":
                 return util.format_size(self("~#filesize", 0))
@@ -344,8 +459,8 @@ class AudioFile(dict, ImageContainer):
                 key = "~" + key
                 if key in self:
                     return self[key]
-                elif key in INTERN_NUM_DEFAULT:
-                    return dict.get(self, key, 0)
+                elif key in NUMERIC_ZERO_DEFAULT:
+                    return 0
                 else:
                     try:
                         val = self[key[2:]]
@@ -365,8 +480,8 @@ class AudioFile(dict, ImageContainer):
             title = dict.get(self, "title")
             if title is None:
                 basename = self("~basename")
-                basename = basename.decode(const.FSCODING, "replace")
-                return "%s [%s]" % (basename, _("Unknown"))
+                return "%s [%s]" % (
+                    decode_value("~basename", basename), _("Unknown"))
             else:
                 return title
         elif key in SORT_TO_TAG:
@@ -381,10 +496,18 @@ class AudioFile(dict, ImageContainer):
 
         role_map = {}
         for key in role_tag_keys:
-            if key != role_tag:
-                role = key.rsplit(":", 1)[-1]
-                for name in self.list(key):
-                    role_map.setdefault(name, []).append(role)
+            if key == role_tag:
+                # #2986: don't add a role description for the bare tag, unless
+                # this is a composite tag (e.g. only show "(Performance)" for
+                # ~people:roles and not ~performer:roles).
+                if sub_keys is None:
+                    continue
+                else:
+                    role = TAG_ROLES.get(role_tag, role_tag)
+            else:
+                role = key.split(":", 1)[-1]
+            for name in self.list(key):
+                role_map.setdefault(name, []).append(role)
 
         if sub_keys is None:
             names = self.list_unique(role_tag_keys)
@@ -408,76 +531,224 @@ class AudioFile(dict, ImageContainer):
 
     @property
     def lyric_filename(self):
-        """Returns the (potential) lyrics filename for this file"""
+        """Returns the validated, or default, lyrics filename for this
+        file. User defined '[memory] lyric_rootpaths' and
+        '[memory] lyric_filenames' matches take precedence"""
 
-        filename = self.comma("title").replace(u'/', u'')[:128] + u'.lyric'
-        sub_dir = ((self.comma("lyricist") or self.comma("artist"))
-                  .replace(u'/', u'')[:128])
+        from quodlibet.pattern \
+            import ArbitraryExtensionFileFromPattern as expand_patterns
 
-        if os.name == "nt":
-            # this was added at a later point. only use escape_filename here
-            # to keep the linux case the same as before
-            filename = escape_filename(filename)
-            sub_dir = escape_filename(sub_dir)
-        else:
-            filename = fsnative(filename)
-            sub_dir = fsnative(sub_dir)
+        rx_params = re.compile(r'[^\\]<[^' + re.escape(os.sep) + r']*[^\\]>')
 
-        path = os.path.join(
-            expanduser(fsnative(u"~/.lyrics")), sub_dir, filename)
-        return path
+        def expand_pathfile(rpf):
+            """Return the expanded RootPathFile"""
+            expanded = []
+            root = expanduser(rpf.root)
+            pathfile = expanduser(rpf.pathfile)
+            if rx_params.search(pathfile):
+                root = expand_patterns(root).format(self)
+                pathfile = expand_patterns(pathfile).format(self)
+            rpf = RootPathFile(root, pathfile)
+            expanded.append(rpf)
+            if not os.path.exists(pathfile) and is_windows():
+                # prioritise a special character encoded version
+                #
+                # most 'alien' chars are supported for 'nix fs paths, and we
+                # only pass the proposed path through 'escape_filename' (which
+                # apparently doesn't respect case) if we don't care about case!
+                #
+                # FIX: assumes 'nix build used on a case-sensitive fs, nt case
+                # insensitive. clearly this is not biting anyone though (yet!)
+                pathfile = os.path.sep.join([rpf.root, rpf.end_escaped])
+                rpf = RootPathFile(rpf.root, pathfile)
+                expanded.insert(len(expanded) - 1, rpf)
+            return expanded
+
+        def sanitise(sep, parts):
+            """Return a santisied version of a path's parts"""
+            return sep.join(part.replace(os.path.sep, u'')[:128]
+                                for part in parts)
+
+        # setup defaults (user-defined take precedence)
+        # root search paths
+        lyric_paths = \
+            config.getstringlist("memory", "lyric_rootpaths", [])
+        # ensure default paths
+        lyric_paths.append(os.path.join(get_home_dir(), ".lyrics"))
+        lyric_paths.append(
+            os.path.join(os.path.dirname(self.comma('~filename'))))
+        # search pathfile names
+        lyric_filenames = \
+            config.getstringlist("memory", "lyric_filenames", [])
+        # ensure some default pathfile names
+        lyric_filenames.append(
+            sanitise(os.sep, [(self.comma("lyricist") or
+                              self.comma("artist")),
+                              self.comma("title")]) + u'.lyric')
+        lyric_filenames.append(
+            sanitise(' - ', [(self.comma("lyricist") or
+                             self.comma("artist")),
+                             self.comma("title")]) + u'.lyric')
+
+        # generate all potential paths (unresolved/unexpanded)
+        pathfiles = OrderedDict()
+        for r in lyric_paths:
+            for f in lyric_filenames:
+                pathfile = os.path.join(r, os.path.dirname(f),
+                                        fsnative(os.path.basename(f)))
+                rpf = RootPathFile(r, pathfile)
+                if not pathfile in pathfiles:
+                    pathfiles[pathfile] = rpf
+
+        #print_d("searching for lyrics in:\n%s" % '\n'.join(pathfiles.keys()))
+
+        # expand each raw pathfile in turn and test for existence
+        match_ = ""
+        pathfiles_expanded = OrderedDict()
+        for pf, rpf in pathfiles.items():
+            for rpf in expand_pathfile(rpf):  # resolved as late as possible
+                pathfile = rpf.pathfile
+                pathfiles_expanded[pathfile] = rpf
+                if os.path.exists(pathfile):
+                    match_ = pathfile
+                    break
+            if match_ != "":
+                break
+
+        if not match_:
+            # search even harder!
+            lyric_extensions = ['lyric', 'lyrics', '', 'txt']
+            #print_d("extending search to extensions: %s" % lyric_extensions)
+
+            def generate_mod_ext_paths(pathfile):
+                # separate pathfile's extension (if any)
+                ext = os.path.splitext(pathfile)[1][1:]
+                path = pathfile[:-1 * len(ext)].strip('.') if ext else pathfile
+                # skip the proposed lyric extension if it is the same as
+                # the original for a given search pathfile stub - it has
+                # already been tested without success!
+                extra_extensions = [x for x in lyric_extensions if x != ext]
+
+                # join valid new extensions to pathfile stub and return
+                return ['.'.join([path, ext]) if ext else path
+                           for ext in extra_extensions]
+
+            # look for a match by modifying the extension for each of the
+            # (now fully resolved) 'pathfiles_expanded' search items
+            for pathfile in pathfiles_expanded.keys():
+                # get alternatives for existence testing
+                paths_mod_ext = generate_mod_ext_paths(pathfile)
+                for path_ext in paths_mod_ext:
+                    if os.path.exists(path_ext):
+                        # persistence has paid off!
+                        #print_d("extended search match!")
+                        match_ = path_ext
+                        break
+                if match_:
+                    break
+
+        if not match_:
+            # default
+            match_ = list(pathfiles_expanded.keys())[0]
+
+        return match_
+
+    @property
+    def has_rating(self):
+        """True if the song has a rating set.
+
+        In case this is False song('~#rating') would return the default value
+        """
+
+        return self.get("~#rating") is not None
+
+    def remove_rating(self):
+        """Removes the set rating so the default will be returned"""
+
+        self.pop("~#rating", None)
 
     def comma(self, key):
         """Get all values of a tag, separated by commas. Synthetic
-        tags are supported, but will be slower. If the value is
-        numeric, that is returned rather than a list."""
+        tags are supported, but will be slower. All list items
+        will be unicode.
+
+        If the value is numeric, that is returned rather than a list.
+        """
 
         if "~" in key or key == "title":
-            v = self(key, u"")
+            if key in FILESYSTEM_TAGS:
+                v = fsn2text(self(key, fsnative()))
+            else:
+                v = self(key, u"")
         else:
             v = self.get(key, u"")
-        if isinstance(v, (int, long, float)):
+
+        if isinstance(v, (int, float)):
             return v
         else:
             return v.replace("\n", ", ")
 
     def list(self, key):
         """Get all values of a tag, as a list. Synthetic tags are supported,
-        but will be slower. Numeric tags are not supported.
+        but will be slower. Numeric tags will give their one value.
+
+        For file path keys the returned list might contain path items
+        (non-unicode).
 
         An empty synthetic tag cannot be distinguished from a non-existent
-        synthetic tag; both result in []."""
+        synthetic tag; both result in [].
+        """
 
         if "~" in key or key == "title":
-            v = self(key, connector="\n")
+            v = self(key)
             if v == "":
                 return []
             else:
-                return v.split("\n")
+                return v.split("\n") if isinstance(v, str) else [v]
         else:
             v = self.get(key)
             return [] if v is None else v.split("\n")
 
-    def list_separate(self, key, connector=" - "):
-        """Similar to list, but will return a list of all combinations
-        for tied tags instead of one comma separated string"""
-        if key[:1] == "~" and "~" in key[1:]:
-            vals = \
-                filter(None,
-                map(lambda x: isinstance(x, basestring) and x or str(x),
-                map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
-                (self(tag) for tag in util.tagsplit(key)))))
-            vals = (val.split("\n") for val in vals)
-            r = [[]]
-            for x in vals:
-                r = [i + [y] for y in x for i in r]
-            return map(connector.join, r)
+    def list_sort(self, key):
+        """Like list but return display,sort pairs when appropriate
+        and work on all tags.
+
+        In case no sort value exists the display one is returned. The sort
+        value is only an empty string if the display one is empty as well.
+        """
+
+        display = decode_value(key, self(key))
+        display = display.split("\n") if display else []
+        sort = []
+        if key in TAG_TO_SORT:
+            sort = decode_value(TAG_TO_SORT[key],
+                                self(TAG_TO_SORT[key]))
+            # it would be better to use something that doesn't fall back
+            # to the key itself, but what?
+            sort = sort.split("\n") if sort else []
+
+        result = []
+        for d, s in zip_longest(display, sort):
+            if d is not None:
+                result.append((d, (s if s is not None and s != "" else d)))
+        return result
+
+    def list_separate(self, key):
+        """For tied tags return the list union of the display,sort values
+           otherwise just do list_sort
+        """
+        if key[:1] == "~" and "~" in key[1:]: # tied tag
+            vals = [self.list_sort(tag) for tag in util.tagsplit(key)]
+            r = [j for i in vals for j in i]
+            return r
         else:
-            return self.list(key)
+            return self.list_sort(key)
 
     def list_unique(self, keys):
         """Returns a combined value of all values in keys; duplicate values
         will be ignored.
+
+        Returns the same as list().
         """
 
         l = []
@@ -498,7 +769,7 @@ class AudioFile(dict, ImageContainer):
 
         merged = AudioFile()
         text = {}
-        for key, value in self.iteritems():
+        for key, value in self.items():
             lower = key.lower()
             if key.startswith("~#"):
                 merged[lower] = value
@@ -522,7 +793,7 @@ class AudioFile(dict, ImageContainer):
     def mounted(self):
         """Return true if the disk the file is on is mounted, or
         the file is not on a disk."""
-        return os.path.ismount(self.get("~mountpoint", "/"))
+        return ismount(self.get("~mountpoint", "/"))
 
     def can_multiple_values(self, key=None):
         """If no arguments are given, return a list of tags that can
@@ -545,12 +816,7 @@ class AudioFile(dict, ImageContainer):
         if k is None:
             return True
 
-        try:
-            if isinstance(k, unicode):
-                k = k.encode("ascii")
-            else:
-                k.decode("ascii")
-        except UnicodeError:
+        if not isascii(k):
             return False
 
         if not k or "=" in k or "~" in k:
@@ -577,45 +843,20 @@ class AudioFile(dict, ImageContainer):
 
         self.sanitize(newname)
 
-    def website(self):
-        """Look for a URL in the audio metadata, or a Google search
-        if no URL can be found."""
-
-        if "website" in self:
-            return self.list("website")[0]
-        for cont in self.list("contact") + self.list("comment"):
-            c = cont.lower()
-            if (c.startswith("http://") or c.startswith("https://") or
-                    c.startswith("www.")):
-                return cont
-            elif c.startswith("//www."):
-                return "http:" + cont
-        else:
-            text = "http://www.google.com/search?q="
-            esc = lambda c: ord(c) > 127 and '%%%x' % ord(c) or c
-            if "labelid" in self:
-                text += ''.join(map(esc, self["labelid"]))
-            else:
-                artist = util.escape("+".join(self("artist").split()))
-                album = util.escape("+".join(self("album").split()))
-                artist = encode(artist)
-                album = encode(album)
-                artist = "%22" + ''.join(map(esc, artist)) + "%22"
-                album = "%22" + ''.join(map(esc, album)) + "%22"
-                text += artist + "+" + album
-            text += "&ie=UTF8"
-            return text
-
     def sanitize(self, filename=None):
         """Fill in metadata defaults. Find ~mountpoint, ~#mtime, ~#filesize
-        and ~#added. Check for null bytes in tags."""
+        and ~#added. Check for null bytes in tags.
+
+        Does not raise.
+        """
 
         # Replace nulls with newlines, trimming zero-length segments
-        for key, val in self.items():
-            if isinstance(val, basestring) and '\0' in val:
+        for key, val in list(self.items()):
+            self[key] = val
+            if isinstance(val, str) and '\0' in val:
                 self[key] = '\n'.join(filter(lambda s: s, val.split('\0')))
             # Remove unnecessary defaults
-            if key in INTERN_NUM_DEFAULT and val == 0:
+            if key in NUMERIC_ZERO_DEFAULT and val == 0:
                 del self[key]
 
         if filename:
@@ -623,7 +864,7 @@ class AudioFile(dict, ImageContainer):
         elif "~filename" not in self:
             raise ValueError("Unknown filename!")
 
-        assert is_fsnative(self["~filename"])
+        assert isinstance(self["~filename"], fsnative)
 
         if self.is_file:
             self["~filename"] = normalize_path(
@@ -634,8 +875,8 @@ class AudioFile(dict, ImageContainer):
                 head, tail = os.path.split(head)
                 # Prevent infinite loop without a fully-qualified filename
                 # (the unit tests use these).
-                head = head or "/"
-                if os.path.ismount(head):
+                head = head or fsnative(u"/")
+                if ismount(head):
                     self["~mountpoint"] = head
         else:
             self["~mountpoint"] = fsnative(u"/")
@@ -662,38 +903,58 @@ class AudioFile(dict, ImageContainer):
             self["~#mtime"] = 0
 
     def to_dump(self):
-        """A string of 'key=value' lines, similar to vorbiscomment output."""
+        """A string of 'key=value' lines, similar to vorbiscomment output.
+
+        Returns:
+            bytes
+        """
+
+        def encode_key(k):
+            return encode(k) if isinstance(k, str) else k
+
         s = []
         for k in self.keys():
-            k = str(k)
-            if isinstance(self[k], int) or isinstance(self[k], long):
-                s.append("%s=%d" % (k, self[k]))
+            enc_key = encode_key(k)
+            assert isinstance(enc_key, bytes)
+
+            if isinstance(self[k], int):
+                l = enc_key + encode("=%d" % self[k])
+                s.append(l)
             elif isinstance(self[k], float):
-                s.append("%s=%f" % (k, self[k]))
+                l = enc_key + encode("=%f" % self[k])
+                s.append(l)
             else:
                 for v2 in self.list(k):
-                    if isinstance(v2, str):
-                        s.append("%s=%s" % (k, v2))
-                    else:
-                        s.append("%s=%s" % (k, encode(v2)))
-        for k in (INTERN_NUM_DEFAULT - set(self.keys())):
-            s.append("%s=%d" % (k, self.get(k, 0)))
+                    if not isinstance(v2, bytes):
+                        v2 = encode(v2)
+                    s.append(enc_key + b"=" + v2)
+        for k in (NUMERIC_ZERO_DEFAULT - set(self.keys())):
+            enc_key = encode_key(k)
+            l = enc_key + encode("=%d" % self.get(k, 0))
+            s.append(l)
         if "~#rating" not in self:
-            s.append("~#rating=%f" % self("~#rating"))
-        s.append("~format=%s" % self.format)
-        s.append("")
-        return "\n".join(s)
+            s.append(encode("~#rating=%f" % self("~#rating")))
+        s.append(encode("~format=%s" % self.format))
+        s.append(b"")
+        return b"\n".join(s)
 
     def from_dump(self, text):
-        """Parses the text created with to_dump and adds the found tags."""
-        for line in text.split("\n"):
+        """Parses the text created with to_dump and adds the found tags.
+
+        Args:
+            text (bytes)
+        """
+
+        for line in text.split(b"\n"):
             if not line:
                 continue
-            parts = line.split("=")
-            key = parts[0]
-            val = "=".join(parts[1:])
+            parts = line.split(b"=")
+            key = decode(parts[0])
+            val = b"=".join(parts[1:])
             if key == "~format":
                 pass
+            elif key in FILESYSTEM_TAGS:
+                self.add(key, bytes2fsn(val, "utf-8"))
             elif key.startswith("~#"):
                 try:
                     self.add(key, int(val))
@@ -703,7 +964,7 @@ class AudioFile(dict, ImageContainer):
                     except ValueError:
                         pass
             else:
-                self.add(key, val)
+                self.add(key, decode(val))
 
     def change(self, key, old_value, new_value):
         """Change 'old_value' to 'new_value' for the given metadata key.
@@ -726,21 +987,24 @@ class AudioFile(dict, ImageContainer):
         else:
             self[key] += "\n" + value
 
-    def remove(self, key, value):
-        """Remove a value from the given key; if the value is not found,
-        remove all values for that key."""
+    def remove(self, key, value=None):
+        """Remove a value from the given key.
+
+        If value is None remove all values for that key, if it exists.
+        If the key or value is not found do nothing.
+        """
+
         if key not in self:
             return
-        elif self[key] == value:
-            del(self[key])
+        elif value is None or self[key] == value:
+            del self[key]
         else:
             try:
                 parts = self.list(key)
                 parts.remove(value)
-                self[key] = "\n".join(parts)
+                self[key] = u"\n".join(parts)
             except ValueError:
-                if key in self:
-                    del(self[key])
+                pass
 
     def replay_gain(self, profiles, pre_amp_gain=0, fallback_gain=0):
         """Return the computed Replay Gain scale factor.
@@ -761,18 +1025,31 @@ class AudioFile(dict, ImageContainer):
                 continue
             else:
                 db += pre_amp_gain
-                scale = 10. ** (db / 20)
-                if scale * peak > 1:
-                    scale = 1.0 / peak  # don't clip
+                try:
+                    scale = 10. ** (db / 20)
+                except OverflowError:
+                    scale = 1.0 / peak
+                else:
+                    if scale * peak > 1:
+                        scale = 1.0 / peak  # don't clip
                 return min(15, scale)
         else:
-            scale = 10. ** ((fallback_gain + pre_amp_gain) / 20)
-            if scale > 1:
-                scale = 1.0  # don't clip
+            try:
+                scale = 10. ** ((fallback_gain + pre_amp_gain) / 20)
+            except OverflowError:
+                scale = 1.0
+            else:
+                if scale > 1:
+                    scale = 1.0  # don't clip
             return min(15, scale)
 
     def write(self):
-        """Write metadata back to the file."""
+        """Write metadata back to the file.
+
+        Raises:
+            AudioFileError: in case writing fails
+        """
+
         raise NotImplementedError
 
     @property
@@ -823,7 +1100,7 @@ class AudioFile(dict, ImageContainer):
 
 # Looks like the real thing.
 DUMMY_SONG = AudioFile({
-    '~#length': 234, '~filename': '/dev/null',
+    '~#length': 234, '~filename': devnull,
     'artist': 'The Artist', 'album': 'An Example Album',
     'title': 'First Track', 'tracknumber': 1,
     'date': '2010-12-31',
