@@ -2,18 +2,23 @@
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 
 from gi.repository import Gtk, GObject, Pango
+from senf import fsnative
 
+from quodlibet import ngettext, _
 from quodlibet import config
-from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
+from quodlibet import app
 
+from quodlibet.qltk.appwindow import AppWindow
+from quodlibet.formats import AudioFileError
 from quodlibet.plugins import PluginManager
 from quodlibet.qltk.delete import trash_files, TrashMenuItem
 from quodlibet.qltk.edittags import EditTags
@@ -22,18 +27,22 @@ from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
+from quodlibet.qltk.menubutton import MenuButton
 from quodlibet.qltk.entry import UndoEntry
-from quodlibet.qltk.about import AboutExFalso
+from quodlibet.qltk.about import AboutDialog
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
 from quodlibet.qltk.x import Align, SeparatorMenuItem, ConfigRHPaned, \
-    Button
+    Button, SymbolicIconImage, MenuItem
 from quodlibet.qltk.window import PersistentWindowMixin, Window, UniqueWindow
-from quodlibet.qltk import icons
+from quodlibet.qltk.msg import CancelRevertSave
+from quodlibet.qltk import Icons
+from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.path import mtime, normalize_path
-from quodlibet.util import connect_obj, connect_destroy
+from quodlibet.util import connect_obj, connect_destroy, format_int_locale
+from quodlibet.update import UpdateDialog
 
 
-class ExFalsoWindow(Window, PersistentWindowMixin):
+class ExFalsoWindow(Window, PersistentWindowMixin, AppWindow):
 
     __gsignals__ = {
         'changed': (GObject.SignalFlags.RUN_LAST, None, (object,)),
@@ -63,36 +72,58 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
 
         bbox = Gtk.HBox(spacing=6)
 
-        about = Gtk.Button()
-        about.add(Gtk.Image.new_from_stock(
-            Gtk.STOCK_ABOUT, Gtk.IconSize.BUTTON))
-        connect_obj(about, 'clicked', self.__show_about, self)
-        bbox.pack_start(about, False, True, 0)
-
-        prefs = Gtk.Button()
-        prefs.add(Gtk.Image.new_from_stock(
-            Gtk.STOCK_PREFERENCES, Gtk.IconSize.BUTTON))
-
-        def prefs_cb(button):
+        def prefs_cb(*args):
             window = PreferencesWindow(self)
             window.show()
-        prefs.connect('clicked', prefs_cb)
-        bbox.pack_start(prefs, False, True, 0)
 
-        plugins = qltk.Button(_("_Plugins"), Gtk.STOCK_EXECUTE)
-
-        def plugin_window_cb(button):
+        def plugin_window_cb(*args):
             window = PluginWindow(self)
             window.show()
-        plugins.connect('clicked', plugin_window_cb)
-        bbox.pack_start(plugins, False, True, 0)
+
+        def about_cb(*args):
+            about = AboutDialog(self, app)
+            about.run()
+            about.destroy()
+
+        def update_cb(*args):
+            d = UpdateDialog(self)
+            d.run()
+            d.destroy()
+
+        menu = Gtk.Menu()
+
+        about_item = MenuItem(_("_About"), Icons.HELP_ABOUT)
+        about_item.connect("activate", about_cb)
+        menu.append(about_item)
+
+        check_item = MenuItem(_("_Check for Updates…"), Icons.NETWORK_SERVER)
+        check_item.connect("activate", update_cb)
+        menu.append(check_item)
+
+        menu.append(SeparatorMenuItem())
+
+        plugin_item = MenuItem(_("_Plugins"), Icons.SYSTEM_RUN)
+        plugin_item.connect("activate", plugin_window_cb)
+        menu.append(plugin_item)
+
+        pref_item = MenuItem(_("_Preferences"), Icons.PREFERENCES_SYSTEM)
+        pref_item.connect("activate", prefs_cb)
+        menu.append(pref_item)
+
+        menu.show_all()
+
+        menu_button = MenuButton(
+                SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.BUTTON),
+                arrow=True, down=False)
+        menu_button.set_menu(menu)
+        bbox.pack_start(menu_button, False, True, 0)
 
         l = Gtk.Label()
         l.set_alignment(1.0, 0.5)
         l.set_ellipsize(Pango.EllipsizeMode.END)
         bbox.pack_start(l, True, True, 0)
 
-        fs = MainFileSelector()
+        self._fs = fs = MainFileSelector()
 
         vb.pack_start(fs, True, True, 0)
         vb.pack_start(Align(bbox, border=6), False, True, 0)
@@ -107,9 +138,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
             page = Page(self, self.__library)
             page.show()
             nb.append_page(page)
-        align = Align(nb, top=3)
-        align.show()
-        hp.pack2(align, resize=True, shrink=False)
+        hp.pack2(nb, resize=True, shrink=False)
         fs.connect('changed', self.__changed, l)
         if dir:
             fs.go_to(dir)
@@ -130,30 +159,38 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         self.get_child().show()
 
         self.__ag = Gtk.AccelGroup()
-        key, mod = Gtk.accelerator_parse("<control>Q")
+        key, mod = Gtk.accelerator_parse("<Primary>Q")
         self.__ag.connect(key, mod, 0, lambda *x: self.destroy())
         self.add_accel_group(self.__ag)
 
+        # GtkosxApplication assumes the menu bar is mapped, so add
+        # it but don't show it.
+        self._dummy_osx_menu_bar = Gtk.MenuBar()
+        vb.pack_start(self._dummy_osx_menu_bar, False, False, 0)
+
     def __library_changed(self, library, songs, fs):
         fs.rescan()
 
     def set_as_osx_window(self, osx_app):
-        osx_app.set_menu_bar(Gtk.MenuBar())
+        osx_app.set_menu_bar(self._dummy_osx_menu_bar)
 
-    def get_osx_is_persistent(self):
+    def get_is_persistent(self):
         return False
 
-    def __show_about(self, window):
-        about = AboutExFalso(self)
-        about.run()
-        about.destroy()
+    def open_file(self, filename):
+        assert isinstance(filename, fsnative)
+
+        if not os.path.isdir(filename):
+            return False
+
+        self._fs.go_to(filename)
 
     def set_pending(self, button, *excess):
         self.__save = button
 
     def __pre_selection_changed(self, view, event, fs, nb):
         if self.__save:
-            resp = qltk.CancelRevertSave(self).run()
+            resp = CancelRevertSave(self).run()
             if resp == Gtk.ResponseType.YES:
                 self.__save.clicked()
             elif resp == Gtk.ResponseType.NO:
@@ -201,34 +238,35 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
             count = len(model or [])
         else:
             count = len(rows)
-        label.set_text(ngettext("%d song", "%d songs", count) % count)
+        label.set_text(numeric_phrase("%d song", "%d songs", count))
 
         for row in rows:
             filename = model[row][0]
             if not os.path.exists(filename):
                 pass
             elif filename in self.__library:
-                file = self.__library[filename]
-                if file("~#mtime") + 1. < mtime(filename):
+                song = self.__library[filename]
+                if song("~#mtime") + 1. < mtime(filename):
                     try:
-                        file.reload()
-                    except StandardError:
+                        song.reload()
+                    except AudioFileError:
                         pass
-                files.append(file)
+                files.append(song)
             else:
                 files.append(formats.MusicFile(filename))
-        files = filter(None, files)
+        files = list(filter(None, files))
         if len(files) == 0:
             self.set_title("Ex Falso")
         elif len(files) == 1:
             self.set_title("%s - Ex Falso" % files[0].comma("title"))
         else:
+            params = ({'title': files[0].comma("title"),
+                       'count': format_int_locale(len(files) - 1)})
             self.set_title(
                 "%s - Ex Falso" %
-                (ngettext("%(title)s and %(count)d more",
-                          "%(title)s and %(count)d more",
-                          len(files) - 1) % (
-                {'title': files[0].comma("title"), 'count': len(files) - 1})))
+                (ngettext("%(title)s and %(count)s more",
+                          "%(title)s and %(count)s more", len(files) - 1)
+                 % params))
         self.__library.add(files)
         self.emit('changed', files)
 
@@ -256,7 +294,7 @@ class PreferencesWindow(UniqueWindow):
         vbox.pack_start(hb, False, True, 0)
         f = qltk.Frame(_("Tag Editing"), child=vbox)
 
-        close = Button(_("_Close"), icons.WINDOW_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         connect_obj(close, 'clicked', lambda x: x.destroy(), self)
         button_box = Gtk.HButtonBox()
         button_box.set_layout(Gtk.ButtonBoxStyle.END)
@@ -276,4 +314,4 @@ class PreferencesWindow(UniqueWindow):
         config.set(section, name, entry.get_text())
 
     def __destroy(self):
-        config.write(const.CONFIG)
+        config.save()
